
#include "BPNGLoggerDetector.hh"
#include <time.h>

using namespace std;

// Constructor
BPNGLoggerDetector::BPNGLoggerDetector()
{
}

// Returns a vector of BP2Devices, search the network for searchTimeOut seconds
std::vector<BP2Device> BPNGLoggerDetector::getLoggerList(unsigned searchTimeOut)
{
	m_loggerList.clear();
	
	IBPNGClient* client = getBPNGClient();
	client->addListener(this);

	time_t start = time(0);

	if (searchTimeOut == 0)
	{
		// scan only once
		client->scanNetworkForLogger();
	}
	else
	{
		while (time(0) - start < searchTimeOut)
			client->scanNetworkForLogger();
	}
	
	client->release();
	return m_loggerList;
}

// listener function for observing blue PiraT 2 devices in network
void WINAPI BPNGLoggerDetector::onBPNGDeviceDetected(OnlineLoggerInfo* info)
{
	BP2Device newDevice(info);

	bool alreadyIn = false;
	for (size_t i = 0; i < m_loggerList.size(); ++i)
	{
		if (m_loggerList[i].mainboardNr == newDevice.mainboardNr)
			alreadyIn = true;
	}

	if (!alreadyIn)
		m_loggerList.push_back(newDevice);
}

void WINAPI BPNGLoggerDetector::onBPNGDeviceDisappeared(OnlineLoggerInfo* info)
{
	BP2Device device(info);
	for (size_t i = 0; i < m_loggerList.size(); ++i)
	{
		if (m_loggerList[i].mainboardNr == device.mainboardNr)
		{
			m_loggerList.erase(m_loggerList.begin() + i);
			break;
		}
	}
}

void WINAPI BPNGLoggerDetector::onBPNGDeviceStateChange(OnlineLoggerInfo* info)
{
	BP2Device device(info);
	for (size_t i = 0; i < m_loggerList.size(); ++i)
	{
		if (m_loggerList[i].mainboardNr == device.mainboardNr)
		{
			m_loggerList[i] = BP2Device(info);
			break;
		}
	}
}
