
#ifndef __RDBEVENTLIST_HH__
#define __RDBEVENTLIST_HH__

#include <vector>
#include <string>

#include "BPNGDefines.h"

/// Implementation class for a wrapper of IRdbEvent using STL classes
/**
 To achieve a compiler independend interface for the blue PiraT 2 client library
 only pointer to complex objects are returned from some functions. The IRdbEvent
 class is can be wrapped by this class RdbEvent to have access to its members in
 the usual way. You only have to pass a IRdbEvent pointer to the constructor.

\sa IRdbEvent, RdbEventList
*/
struct RdbEvent
{
	RdbEvent(IRdbEvent* rdbEvent)
	{
		type = rdbEvent->getType();
		uniqueID = rdbEvent->getUniqueId();
		timeStamp = rdbEvent->getTimeStamp();
		timeZone = rdbEvent->getTimeZone();
		index = rdbEvent->getIndex();
		comment = rdbEvent->getComment();
	}

	RdbEventType type;
	uint64_t uniqueID;
	uint64_t timeStamp;
	std::string timeZone;
	int index;
	std::string comment;
};

/// Implementation class for a wrapper of IRdbEventList using STL classes
/**
 To achieve a compiler independend interface for the blue PiraT 2 client library
 only pointer to complex objects are returned from some functions. 
 The class IRdbEventList is nothing else than a vector of IRdbEvent objects.
 Pass a pointer to IRdbEventList to the constructor of this wrapper class RdbEventList
 and you get a STL vector of RdbEvent objects which by itself is a wrapper to IRdbEvent
 
 \sa RdbEvent, IRdbEventList, IRdbEvent
*/
class RdbEventList : public std::vector<RdbEvent>
{
public:
	RdbEventList(IRdbEventList* list)
	{
		int size = list->getSize();
		this->reserve(size);
		for (int i = 0; i < size; ++i)
		{
			this->push_back(RdbEvent(list->getEvent(i)));
		}
	}
};

#endif // __RDBEVENTLIST_HH__